# This file is part of pyZohoAPI, Copyright (C) Todd D. Esposito 2021.
# Distributed under the MIT License (see https://opensource.org/licenses/MIT).

__version__ = '0.9.0'
__all__ = [
    "ZohoBooks",
    "ZohoInventory",
]

from .books import ZohoBooks
from .inventory import ZohoInventory
